@@ -245,8 +245,18 @@ public class MozcService extends InputMethodService {
     public void onKeyEvent(
         ProtoCommands.KeyEvent mozcKeyEvent, KeyEventInterface keyEvent,
         KeyboardSpecification keyboardSpecification, List<? extends TouchEvent> touchEventList) {
-      if (mozcKeyEvent == null && keyEvent == null && keyboardSpecification == null) {
-        sessionExecutor.usageStatsEvent(touchEventList);
+      if (mozcKeyEvent == null && keyboardSpecification == null) {
+        // We don't send a key event to Mozc native layer since {@code mozcKeyEvent} is null, and we
+        // don't need to update the keyboard specification since {@code keyboardSpecification} is
+        // also null.
+        if (keyEvent == null) {
+          // Send a usage information to Mozc native layer.
+          sessionExecutor.usageStatsEvent(touchEventList);
+        } else {
+          // Send a key event (which is generated by Mozc in the usual case) to application.
+          Preconditions.checkArgument(touchEventList.isEmpty());
+          sessionExecutor.sendKeyEvent(keyEvent, sendKeyToApplicationCallback);
+        }
         return;
       }
 
@@ -561,7 +571,9 @@ public class MozcService extends InputMethodService {
   @Override
   public void onDestroy() {
     feedbackManager.release();
-    // Unregister this from MozcPreference to help GC.
+    if (sessionExecutor != null) {
+      sessionExecutor.syncData();
+    }
     super.onDestroy();
   }
 
