@@ -44,20 +44,22 @@
 #endif  // OS_MACOSX
 #endif  // OS_WIN
 
-#include <cstdlib>
-#include <cstring>
-#include <sstream>
-#include <string>
 #ifdef OS_MACOSX
 #include <cerrno>
 #endif  // OS_MACOSX
+#include <cstdlib>
+#include <cstring>
+#ifdef OS_WIN
+#include <memory>
+#endif  // OS_WIN
+#include <sstream>
+#include <string>
 
 #include "base/const.h"
 #include "base/file_util.h"
 #include "base/logging.h"
 #include "base/mac_util.h"
 #include "base/number_util.h"
-#include "base/scoped_ptr.h"
 #include "base/singleton.h"
 #include "base/util.h"
 #include "base/win_util.h"
@@ -69,6 +71,10 @@
 #include "base/android_util.h"
 #endif  // OS_ANDROID
 
+#ifdef OS_WIN
+using std::unique_ptr;
+#endif  // OS_WIN
+
 namespace mozc {
 
 namespace {
@@ -120,7 +126,7 @@ class LocalAppDataDirectoryCache {
   }
 
   static HRESULT TryGetLocalAppData(string *dir) {
-    if (dir == NULL) {
+    if (dir == nullptr) {
       return E_FAIL;
     }
     dir->clear();
@@ -144,7 +150,7 @@ class LocalAppDataDirectoryCache {
     // applications, to avoid user profiles from being roamed by indexers.
     wchar_t config[MAX_PATH] = {};
     const HRESULT result = ::SHGetFolderPathW(
-        NULL, CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, &config[0]);
+        nullptr, CSIDL_LOCAL_APPDATA, nullptr, SHGFP_TYPE_CURRENT, &config[0]);
     if (FAILED(result)) {
       return result;
     }
@@ -168,7 +174,7 @@ class LocalAppDataDirectoryCache {
     // TODO(yukawa): Establish more reliable way to obtain the path.
     wchar_t config[MAX_PATH] = {};
     const HRESULT result = ::SHGetFolderPathW(
-        NULL, CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, &config[0]);
+        nullptr, CSIDL_LOCAL_APPDATA, nullptr, SHGFP_TYPE_CURRENT, &config[0]);
     if (FAILED(result)) {
       return result;
     }
@@ -186,7 +192,7 @@ class LocalAppDataDirectoryCache {
   }
 
   static HRESULT TryGetLocalAppDataLow(string *dir) {
-    if (dir == NULL) {
+    if (dir == nullptr) {
       return E_FAIL;
     }
     dir->clear();
@@ -201,7 +207,7 @@ class LocalAppDataDirectoryCache {
     // http://msdn.microsoft.com/en-us/library/bb762584(VS.85).aspx
     // GUID: {A520A1A4-1780-4FF6-BD18-167343C5AF16}
     const HMODULE hLib = WinUtil::LoadSystemLibrary(L"shell32.dll");
-    if (hLib == NULL) {
+    if (hLib == nullptr) {
       return E_NOTIMPL;
     }
 
@@ -209,23 +215,23 @@ class LocalAppDataDirectoryCache {
         const GUID &, DWORD, HANDLE, PWSTR *);
     FPSHGetKnownFolderPath func = reinterpret_cast<FPSHGetKnownFolderPath>
         (::GetProcAddress(hLib, "SHGetKnownFolderPath"));
-    if (func == NULL) {
+    if (func == nullptr) {
       ::FreeLibrary(hLib);
       return E_NOTIMPL;
     }
 
-    wchar_t *task_mem_buffer = NULL;
+    wchar_t *task_mem_buffer = nullptr;
     const HRESULT result =
-        (*func)(FOLDERID_LocalAppDataLow, 0, NULL, &task_mem_buffer);
+        (*func)(FOLDERID_LocalAppDataLow, 0, nullptr, &task_mem_buffer);
     if (FAILED(result)) {
-      if (task_mem_buffer != NULL) {
+      if (task_mem_buffer != nullptr) {
         ::CoTaskMemFree(task_mem_buffer);
       }
       ::FreeLibrary(hLib);
       return result;
     }
 
-    if (task_mem_buffer == NULL) {
+    if (task_mem_buffer == nullptr) {
       ::FreeLibrary(hLib);
       return E_UNEXPECTED;
     }
@@ -364,7 +370,7 @@ class ProgramFilesX86Cache {
   }
 
   static HRESULT TryProgramFilesPath(string *path) {
-    if (path == NULL) {
+    if (path == nullptr) {
       return E_FAIL;
     }
     path->clear();
@@ -376,14 +382,14 @@ class ProgramFilesX86Cache {
     // we should use CSIDL_PROGRAM_FILESX86 to find server, renderer, and other
     // binaries' path.
     const HRESULT result = ::SHGetFolderPathW(
-        NULL, CSIDL_PROGRAM_FILESX86, NULL,
+        nullptr, CSIDL_PROGRAM_FILESX86, nullptr,
         SHGFP_TYPE_CURRENT, program_files_path_buffer);
 #elif defined(_M_IX86)
     // In 32-bit processes (such as server, renderer, and other binaries),
     // CSIDL_PROGRAM_FILES always points 32-bit Program Files directory
     // even if they are running in 64-bit Windows.
     const HRESULT result = ::SHGetFolderPathW(
-        NULL, CSIDL_PROGRAM_FILES, NULL,
+        nullptr, CSIDL_PROGRAM_FILES, nullptr,
         SHGFP_TYPE_CURRENT, program_files_path_buffer);
 #else  // !_M_X64 && !_M_IX86
 #error "Unsupported CPU architecture"
@@ -513,7 +519,7 @@ class UserSidImpl {
 };
 
 UserSidImpl::UserSidImpl() {
-  HANDLE htoken = NULL;
+  HANDLE htoken = nullptr;
   if (!::OpenProcessToken(::GetCurrentProcess(), TOKEN_QUERY, &htoken)) {
     sid_ = SystemUtil::GetUserNameAsString();
     LOG(ERROR) << "OpenProcessToken failed: " << ::GetLastError();
@@ -521,8 +527,8 @@ UserSidImpl::UserSidImpl() {
   }
 
   DWORD length = 0;
-  ::GetTokenInformation(htoken, TokenUser, NULL, 0, &length);
-  scoped_array<char> buf(new char[length]);
+  ::GetTokenInformation(htoken, TokenUser, nullptr, 0, &length);
+  unique_ptr<char[]> buf(new char[length]);
   PTOKEN_USER p_user = reinterpret_cast<PTOKEN_USER>(buf.get());
 
   if (length == 0 ||
@@ -562,14 +568,13 @@ string SystemUtil::GetUserSidAsString() {
 namespace {
 
 string GetObjectNameAsString(HANDLE handle) {
-  if (handle == NULL) {
+  if (handle == nullptr) {
     LOG(ERROR) << "Unknown handle";
     return "";
   }
 
   DWORD size = 0;
-  if (::GetUserObjectInformationA(handle, UOI_NAME,
-                                  NULL, 0, &size) ||
+  if (::GetUserObjectInformationA(handle, UOI_NAME, nullptr, 0, &size) ||
       ERROR_INSUFFICIENT_BUFFER != ::GetLastError()) {
     LOG(ERROR) << "GetUserObjectInformationA() failed: "
                << ::GetLastError();
@@ -581,7 +586,7 @@ string GetObjectNameAsString(HANDLE handle) {
     return "";
   }
 
-  scoped_array<char> buf(new char[size]);
+  unique_ptr<char[]> buf(new char[size]);
   DWORD return_size = 0;
   if (!::GetUserObjectInformationA(handle, UOI_NAME, buf.get(),
                                    size, &return_size)) {
@@ -621,7 +626,8 @@ bool GetCurrentSessionId(uint32 *session_id) {
 // desktop is most appropriate and important desktop for our use case.
 // See http://blogs.adobe.com/asset/2012/10/new-security-capabilities-in-adobe-reader-and-acrobat-xi-now-available.html
 string GetInputDesktopName() {
-  const HDESK desktop_handle = OpenInputDesktop(0, FALSE, DESKTOP_READOBJECTS);
+  const HDESK desktop_handle =
+      ::OpenInputDesktop(0, FALSE, DESKTOP_READOBJECTS);
   if (desktop_handle == nullptr) {
     return "";
   }
@@ -755,7 +761,7 @@ typedef IsWindowsVerXOrLaterCache<6, 3> IsWindows8_1OrLaterCache;
 // TODO(yukawa): Use API wrapper so that unit test can emulate any case.
 class SystemDirectoryCache {
  public:
-  SystemDirectoryCache() : system_dir_(NULL) {
+  SystemDirectoryCache() : system_dir_(nullptr) {
     const UINT copied_len_wo_null_if_success =
         ::GetSystemDirectory(path_buffer_, arraysize(path_buffer_));
     if (copied_len_wo_null_if_success >= arraysize(path_buffer_)) {
@@ -766,7 +772,7 @@ class SystemDirectoryCache {
     system_dir_ = path_buffer_;
   }
   const bool succeeded() const {
-    return system_dir_ != NULL;
+    return system_dir_ != nullptr;
   }
   const wchar_t *system_dir() const {
     return system_dir_;
@@ -1048,8 +1054,8 @@ const wchar_t *SystemUtil::GetSystemDir() {
   return Singleton<SystemDirectoryCache>::get()->system_dir();
 }
 
-bool SystemUtil::GetFileVersion(const wstring &file_fullpath,
-                          int *major, int *minor, int *build, int *revision) {
+bool SystemUtil::GetFileVersion(const wstring &file_fullpath, int *major,
+                                int *minor, int *build, int *revision) {
   DCHECK(major);
   DCHECK(minor);
   DCHECK(build);
@@ -1074,7 +1080,7 @@ bool SystemUtil::GetFileVersion(const wstring &file_fullpath,
     return false;
   }
 
-  scoped_array<BYTE> version_buffer(new BYTE[version_size]);
+  unique_ptr<BYTE[]> version_buffer(new BYTE[version_size]);
 
   if (!::GetFileVersionInfoW(file_fullpath.c_str(), 0,
                              version_size, version_buffer.get())) {
@@ -1083,7 +1089,7 @@ bool SystemUtil::GetFileVersion(const wstring &file_fullpath,
     return false;
   }
 
-  VS_FIXEDFILEINFO *fixed_fileinfo = NULL;
+  VS_FIXEDFILEINFO *fixed_fileinfo = nullptr;
   UINT length = 0;
   if (!::VerQueryValueW(version_buffer.get(), L"\\",
                         reinterpret_cast<LPVOID *>(&fixed_fileinfo),
