@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -36,19 +36,27 @@
 #include <QtGui/QSplitterHandle>
 #include <QtGui/QSplitter>
 
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "gui/dictionary_tool/ui_dictionary_tool.h"
-#include "dictionary/user_dictionary_storage.h"
 #include "dictionary/user_dictionary_importer.h"
 
 namespace mozc {
 
+class UserPOSInterface;
+
 namespace client {
-class Session;
-}
+class ClientInterface;
+}  // namespace client
+
+namespace user_dictionary {
+class UserDictionarySession;
+}  // namespace user_dictionary
 
 namespace gui {
 
 class ImportDialog;
+class FindDialog;
 
 class DictionaryTool : public QMainWindow,
                        private Ui::DictionaryTool {
@@ -72,9 +80,9 @@ class DictionaryTool : public QMainWindow,
 
   bool eventFilter(QObject *obj, QEvent *event);
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   bool winEvent(MSG * message, long * result);
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 
  private slots:
   void CreateDictionary();
@@ -112,6 +120,9 @@ class DictionaryTool : public QMainWindow,
   // Returns information on the current dictionary.
   DictionaryInfo current_dictionary() const;
 
+  // Returns true if the current dictionary is sync dictionary.
+  bool IsCurrentDictionaryForSync() const;
+
   // Save content of the current dictionary to the UserDictionaryStorage
   void SyncToStorage();
 
@@ -158,13 +169,32 @@ class DictionaryTool : public QMainWindow,
   // send Reload command to the server.
   void SaveAndReloadServer();
 
+  // 1. Shows a dialog box and get new |commnet|.
+  // 2. Changes the comemnt of all selected.
+  void EditComment();
+
+  // Changes the POS of all selected items to |pos|.
+  void EditPOS(const string &pos);
+
+  // Moves selected items to the dictionary whose row is |dictionary_row|.
+  void MoveTo(int dictionary_row);
+
   // Helper functions to check if a file with given name is readable
   // to import or writable to export without trying to open it.
   static bool IsWritableToExport(const string &file_name);
   static bool IsReadableToImport(const string &file_name);
 
-  ImportDialog *dialog_;
-  scoped_ptr<UserDictionaryStorage> storage_;
+  // Helper function for DeleteWord and MoveTo.
+  // Fills selected word entry rows as a unique sorted sequence.
+  void GetSortedSelectedRows(vector<int> *rows) const;
+
+  // Returns a pointer to the first selected dictionary.
+  // Returns NULL if no dictionary is selected.
+  QListWidgetItem *GetFirstSelectedDictionary() const;
+
+  ImportDialog *import_dialog_;
+  FindDialog   *find_dialog_;
+  scoped_ptr<mozc::user_dictionary::UserDictionarySession> session_;
 
   // ID of current selected dictionary. This needs to be maintained
   // separate from selection on the list widget because data is saved
@@ -211,6 +241,7 @@ class DictionaryTool : public QMainWindow,
   QAction *new_action_;
   QAction *rename_action_;
   QAction *delete_action_;
+  QAction *find_action_;
   QAction *import_create_action_;
   QAction *import_append_action_;
   QAction *export_action_;
@@ -219,9 +250,14 @@ class DictionaryTool : public QMainWindow,
   // status message
   QString statusbar_message_;
 
-  scoped_ptr<client::Session> session_;
+  scoped_ptr<client::ClientInterface> client_;
 
   bool is_available_;
+
+  // The maximum number of entries for a dictionary currently selected.
+  int max_entry_size_;
+
+  scoped_ptr<const UserPOSInterface> user_pos_;
 };
 }  // namespace gui
 }  // namespace mozc
