@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,34 +29,27 @@
 
 #include "gui/post_install_dialog/post_install_dialog.h"
 
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
 #include <windows.h>
 #endif
 
 #include <QtGui/QtGui>
-#include "base/base.h"
+#include "base/logging.h"
 #include "base/process.h"
 #include "base/run_level.h"
 #include "base/util.h"
-#include "dictionary/user_dictionary_importer.h"
-#include "dictionary/user_dictionary_storage.h"
-#include "dictionary/user_dictionary_util.h"
-#include "gui/base/win_util.h"
+#include "gui/base/setup_util.h"
 #include "usage_stats/usage_stats.h"
 
-#ifdef OS_WINDOWS
-#include "win32/base/imm_util.h"
-#include "win32/base/migration_util.h"
+#ifdef OS_WIN
 #include "base/win_util.h"
+#include "win32/base/imm_util.h"
 #endif
 
 namespace mozc {
 namespace gui {
-
 PostInstallDialog::PostInstallDialog()
-    : storage_(
-          new UserDictionaryStorage(
-              UserDictionaryUtil::GetUserDictionaryFileName())) {
+    : setuputil_(new SetupUtil()) {
   setupUi(this);
   setWindowFlags(Qt::WindowSystemMenuHint |
                  Qt::MSWindowsFixedSizeDialogHint |
@@ -138,15 +131,12 @@ PostInstallDialog::PostInstallDialog()
   }
 
   // set the default state of migrateDefaultIMEUserDictionaryCheckBox
-  const bool status = (!RunLevel::IsElevatedByUAC() && storage_->Lock());
+  const bool status = (!RunLevel::IsElevatedByUAC() &&
+                       setuputil_->LockUserDictionary());
   migrateDefaultIMEUserDictionaryCheckBox->setVisible(status);
 
   // import MS-IME by default
   migrateDefaultIMEUserDictionaryCheckBox->setChecked(true);
-
-#ifdef OS_WINDOWS
-  win32::MigrationUtil::LaunchBrokerForPostInstallCleanup();
-#endif  // OS_WINDOWS
 }
 
 PostInstallDialog::~PostInstallDialog() {
@@ -179,11 +169,11 @@ bool PostInstallDialog::ShowHelpPageIfRequired() {
 void PostInstallDialog::OnLogoffNow() {
   usage_stats::UsageStats::IncrementCount("PostInstallLogoffNowButton");
   ApplySettings();
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   mozc::WinUtil::Logoff();
 #else
   // not supported on Mac and Linux
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
   done(QDialog::Accepted);
 }
 
@@ -205,75 +195,27 @@ void PostInstallDialog::reject() {
 }
 
 void PostInstallDialog::ApplySettings() {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
+  uint32 flags = SetupUtil::NONE;
   if (setAsDefaultCheckBox->isChecked()) {
-    usage_stats::UsageStats::IncrementCount("PostInstallSetDefault");
-    win32::ImeUtil::SetDefault();
-  } else {
-    usage_stats::UsageStats::IncrementCount("PostInstallNotSetDefault");
+    flags |= SetupUtil::IME_DEFAULT;
   }
-
-  if (IMEHotKeyDisabledCheckBox->isEnabled()) {
-    if (!WinUtil::SetIMEHotKeyDisabled(
-            IMEHotKeyDisabledCheckBox->isChecked())) {
-      LOG(ERROR) << "Failed to set IMEHotKey";
-    }
+  if (IMEHotKeyDisabledCheckBox->isEnabled() &&
+      IMEHotKeyDisabledCheckBox->isChecked()) {
+    flags |= SetupUtil::DISABLE_HOTKEY;
   }
-
   if (migrateDefaultIMEUserDictionaryCheckBox->isChecked() &&
       migrateDefaultIMEUserDictionaryCheckBox->isVisible()) {
-    storage_->Load();
-    // create UserDictionary if the current user dictionary is empty
-    if (!storage_->Exists()) {
-      const QString name = tr("User Dictionary 1");
-      uint64 dic_id = 0;
-      if (!storage_->CreateDictionary(name.toStdString(),
-                                      &dic_id)) {
-        LOG(ERROR) << "Failed to create a new dictionary.";
-        return;
-      }
-    }
-
-    // Import MS-IME's dictionary to an unique dictionary labeled
-    // as "MS-IME"
-    uint64 dic_id = 0;
-    const QString name = tr("MS-IME User Dictionary");
-    for (size_t i = 0; i < storage_->dictionaries_size(); ++i) {
-      if (storage_->dictionaries(i).name() == name.toStdString()) {
-        dic_id = storage_->dictionaries(i).id();
-        break;
-      }
-    }
-
-    if (dic_id == 0) {
-      if (!storage_->CreateDictionary(name.toStdString(),
-                                      &dic_id)) {
-        LOG(ERROR) << "Failed to create a new dictionary.";
-        return;
-      }
-    }
-
-    UserDictionaryStorage::UserDictionary *dic =
-        storage_->GetUserDictionary(dic_id);
-    if (dic == NULL) {
-      LOG(ERROR) << "GetUserDictionary returned NULL";
-      return;
-    }
-
-    if (UserDictionaryImporter::ImportFromMSIME(dic) !=
-        UserDictionaryImporter::IMPORT_NO_ERROR) {
-      LOG(ERROR) << "ImportFromMSIME failed";
-    }
-
-    storage_->Save();
+    flags |= SetupUtil::IMPORT_MSIME_DICTIONARY;
   }
+  setuputil_->SetDefaultProperty(flags);
 #else
   // not supported on Mac and Linux
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 void PostInstallDialog::OnsetAsDefaultCheckBoxToggled(int state) {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   // IMEHotKey is only activated when setAsDefaultCheckBox is checked.
   IMEHotKeyDisabledCheckBox->setChecked(state);
   IMEHotKeyDisabledCheckBox->setEnabled(static_cast<bool>(state));
@@ -281,12 +223,12 @@ void PostInstallDialog::OnsetAsDefaultCheckBoxToggled(int state) {
 }
 
 bool PostInstallDialog::IsShowHelpPageRequired() {
-#ifdef OS_WINDOWS
+#ifdef OS_WIN
   return !win32::ImeUtil::IsCtfmonRunning();
 #else
   // not supported on Mac and Linux
   return false;
-#endif  // OS_WINDOWS
+#endif  // OS_WIN
 }
 
 }  // namespace gui
