@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -27,8 +27,8 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-// skip all if Windows
-#ifdef OS_LINUX
+// skip all if non-Linux or Android.
+#if defined(OS_LINUX) && !defined(OS_ANDROID)
 
 #include "ipc/ipc.h"
 
@@ -41,9 +41,6 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/un.h>
-#ifdef OS_MACOSX
-#include <sys/ucred.h>
-#endif
 #include <sys/wait.h>
 #include <unistd.h>
 
@@ -51,12 +48,14 @@
 #include <cstring>
 #include <cstdlib>
 
-#include "base/util.h"
+#include "base/file_util.h"
+#include "base/logging.h"
+#include "base/thread.h"
 #include "ipc/ipc_path_manager.h"
 
 #ifndef UNIX_PATH_MAX
 #define UNIX_PATH_MAX 108
-#endif
+#endif  // UNIX_PATH_MAX
 
 namespace mozc {
 
@@ -65,13 +64,13 @@ namespace {
 const int kInvalidSocket = -1;
 
 void mkdir_p(const string &dirname) {
-  const string parent_dir = mozc::Util::Dirname(dirname);
+  const string parent_dir = FileUtil::Dirname(dirname);
   struct stat st;
   if (!parent_dir.empty() &&
       ::stat(parent_dir.c_str(), &st) < 0) {
     mkdir_p(parent_dir);
   }
-  mozc::Util::CreateDirectory(dirname);
+  FileUtil::CreateDirectory(dirname);
 }
 
 bool IsReadTimeout(int socket, int timeout) {
@@ -124,29 +123,6 @@ bool IsWriteTimeout(int socket, int timeout) {
 bool IsPeerValid(int socket, pid_t *pid) {
   *pid = 0;
 
-#ifdef OS_MACOSX
-  // If the OS is MAC, we should validate the peer by using LOCAL_PEERCRED.
-  struct xucred peer_cred;
-  socklen_t peer_cred_len = sizeof(struct xucred);
-  if (::getsockopt(socket, 0, LOCAL_PEERCRED,
-                   &peer_cred, &peer_cred_len) < 0) {
-    LOG(ERROR) << "cannot get peer credential.  NOT a Unix socket?";
-    return false;
-  }
-  if (peer_cred.cr_version != XUCRED_VERSION) {
-    LOG(WARNING) << "credential version mismatch.";
-    return false;
-  }
-  if (peer_cred.cr_uid != ::geteuid()) {
-    LOG(WARNING) << "uid mismatch." << peer_cred.cr_uid << "!=" << ::geteuid();
-    return false;
-  }
-
-  // MacOS doesn't have cr_pid;
-  *pid = 0;
-#endif
-
-#ifdef OS_LINUX
   // On ARM Linux, we do nothing and just return true since the platform
   // sometimes doesn't support the getsockopt(sock, SOL_SOCKET, SO_PEERCRED)
   // system call.
@@ -168,7 +144,6 @@ bool IsPeerValid(int socket, pid_t *pid) {
 
   *pid = peer_cred.pid;
 #endif  // __arm__
-#endif
 
   return true;
 }
@@ -184,13 +159,7 @@ bool SendMessage(int socket,
       *last_ipc_error = IPC_TIMEOUT_ERROR;
       return false;
     }
-    const ssize_t l = ::send(socket, buf, buf_length_left,
-#ifdef OS_MACOSX
-                             SO_NOSIGPIPE
-#else
-                             MSG_NOSIGNAL
-#endif
-                             );
+    const ssize_t l = ::send(socket, buf, buf_length_left, MSG_NOSIGNAL);
     if (l < 0) {
       // An error occurs.
       LOG(ERROR) << "an error occurred during sending \""
@@ -289,7 +258,7 @@ void IPCClient::Init(const string &name, const string &server_path) {
 
   for (size_t trial = 0; trial < 2; ++trial) {
     string server_address;
-    if (!manager->GetPathName(&server_address)) {
+    if (!manager->LoadPathName() || !manager->GetPathName(&server_address)) {
       continue;
     }
     sockaddr_un address;
@@ -309,12 +278,7 @@ void IPCClient::Init(const string &name, const string &server_path) {
     address.sun_family = AF_UNIX;
     ::memcpy(address.sun_path, server_address.data(), server_address_length);
     address.sun_path[server_address_length] = '\0';
-#ifdef OS_MACOSX
-    address.sun_len = SUN_LEN(&address);
-    const size_t sun_len = sizeof(address);
-#else
     const size_t sun_len = sizeof(address.sun_family) + server_address_length;
-#endif
     pid_t pid = 0;
     if (::connect(socket_,
                   reinterpret_cast<const sockaddr*>(&address),
@@ -394,13 +358,18 @@ IPCServer::IPCServer(const string &name,
                      int32 timeout)
     : connected_(false), socket_(kInvalidSocket), timeout_(timeout) {
   IPCPathManager *manager = IPCPathManager::GetIPCPathManager(name);
-  if (!manager->CreateNewPathName() ||
-      !manager->GetPathName(&server_address_)) {
-    LOG(ERROR) << "Cannot make IPC path name";
+  if (!manager->CreateNewPathName() && !manager->LoadPathName()) {
+    LOG(ERROR) << "Cannot prepare IPC path name";
     return;
   }
 
-  const string dirname = Util::Dirname(server_address_);
+  if (!manager->GetPathName(&server_address_)) {
+    LOG(ERROR) << "Cannot make IPC path name";
+    return;
+  }
+  DCHECK(!server_address_.empty());
+
+  const string dirname = FileUtil::Dirname(server_address_);
   mkdir_p(dirname);
 
   sockaddr_un addr;
@@ -429,12 +398,7 @@ IPCServer::IPCServer(const string &name,
                SO_REUSEADDR,
                reinterpret_cast<char *>(&on),
                sizeof(on));
-#ifdef OS_MACOSX
-  addr.sun_len = SUN_LEN(&addr);
-  const size_t sun_len = sizeof(addr);
-#else
   const size_t sun_len = sizeof(addr.sun_family) + server_address_.size();
-#endif
   if (!IsAbstractSocket(server_address_)) {
     // Linux does not use files for IPC.
     ::chmod(server_address_.c_str(), 0600);
@@ -522,6 +486,10 @@ void IPCServer::Loop() {
   socket_ = kInvalidSocket;
 }
 
+void IPCServer::Terminate() {
+  server_thread_->Terminate();
+}
+
 };  // namespace mozc
 
-#endif  // OS_LINUX
+#endif  // OS_LINUX && !OS_ANDROID
