@@ -37,8 +37,8 @@
 
 #include <string>
 
-#include "base/base.h"
 #include "base/const.h"
+#include "base/cpu_stats.h"
 #include "base/logging.h"
 #include "base/mutex.h"
 #include "base/singleton.h"
@@ -69,7 +69,7 @@ FPSetFileCompletionNotificationModes
 #define FILE_SKIP_SET_EVENT_ON_HANDLE 0x2
 #endif  // FILE_SKIP_SET_EVENT_ON_HANDLE
 
-static once_t g_once = MOZC_ONCE_INIT;
+once_t g_once = MOZC_ONCE_INIT;
 
 void InitAPIsForVistaAndLater() {
   // We have to load the function pointer dynamically
@@ -95,6 +95,12 @@ void InitAPIsForVistaAndLater() {
       (::GetProcAddress(lib, "SetFileCompletionNotificationModes"));
 }
 
+size_t GetNumberOfProcessors() {
+  // thread-safety is not required.
+  static size_t num = CPUStats().GetNumberOfProcessors();
+  return max(num, 1);
+}
+
 // Least significant bit of OVERLAPPED::hEvent can be used for special
 // purpose against GetQueuedCompletionStatus API.
 // http://msdn.microsoft.com/en-us/library/windows/desktop/aa364986.aspx
@@ -759,6 +765,17 @@ void IPCClient::Init(const string &name, const string &server_path) {
     wstring wserver_address;
     Util::UTF8ToWide(server_address.c_str(), &wserver_address);
 
+    if (GetNumberOfProcessors() == 1) {
+      // When the code is running in single processor environment, sometimes
+      // IPC server has not finished the clean-up tasks for the previous IPC
+      // session here. So we intentionally call WaitNamedPipe API so that IPC
+      // server has a chance to complete clean-up tasks if necessary.
+      // NOTE: We cannot set 0 for the wait time because 0 has a special meaning
+      // as |NMPWAIT_USE_DEFAULT_WAIT|.
+      const DWORD kMinWaitTimeForWaitNamedPipe = 1;
+      ::WaitNamedPipe(wserver_address.c_str(), kMinWaitTimeForWaitNamedPipe);
+    }
+
     ScopedHandle new_handle(::CreateFile(wserver_address.c_str(),
                                          GENERIC_READ | GENERIC_WRITE,
                                          0, nullptr, OPEN_EXISTING,
