@@ -1,4 +1,4 @@
-// Copyright 2010, Google Inc.
+// Copyright 2010-2013, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,21 +30,17 @@
 #import <InputMethodKit/InputMethodKit.h>
 #import "mac/common.h"
 
-#include "session/commands.pb.h"  // for CompositionMode
+// For mozc::commands::CompositionMode
+#include "renderer/renderer_command.pb.h"
 
 @class NSMenu;
 @class KeyCodeMap;
 
 namespace mozc {
 namespace client {
-class SessionInterface;
+class ClientInterface;
 }  // namespace mozc::client
 
-namespace commands {
-class Output;
-class RendererCommand;
-}
-
 namespace renderer {
 class RendererInterface;
 }  // namespace mozc::renderer
@@ -86,36 +82,55 @@ class RendererInterface;
   // want to call this every key events.
   BOOL checkInputMode_;
 
+  // |suppressSuggestion_| indicates whether to suppress the suggestion.
+  BOOL suppressSuggestion_;
+
   // |keyCodeMap_| manages the mapping between Mac key code and mozc
   // key events.
   KeyCodeMap *keyCodeMap_;
 
   // |clientBundle_| is the Bundle ID of the client application which
   // the controller communicates with.
-  NSString *clientBundle_;
+  string *clientBundle_;
 
   NSRange replacementRange_;
 
+  // |lastKeyDownTime_| and |lastKeyCode_| are used to handle double tapping.
+  NSTimeInterval lastKeyDownTime_;
+  uint16 lastKeyCode_;
+
   // |candidateController_| controls the candidate windows.
   mozc::renderer::RendererInterface *candidateController_;
 
   // |rendererCommand_| stores the command sent to |candidateController_|
   mozc::commands::RendererCommand *rendererCommand_;
 
-  // |session_| manages connection to the mozc server.
-  mozc::client::SessionInterface *session_;
+  // |mozcClient_| manages connection to the mozc server.
+  mozc::client::ClientInterface *mozcClient_;
 
-  // |server_| holds the reference to GoogleJapaneseInputServer.
-  id<ServerCallback> server_;
+  // |imkServer_| holds the reference to GoogleJapaneseInputServer.
+  id<ServerCallback> imkServer_;
+
+  // |imkClientForTest_| holds the reference to the client object for unit test.
+  id imkClientForTest_;
 
   // |menu_| is the NSMenu to be shown in the pulldown menu-list of
   // the IME.
   IBOutlet NSMenu *menu_;
+
+  // |callback_command_| stores the callback message which is recieved from the
+  // server. This callback will be cancelled when the user presses the
+  // subsequent key. In the current implementation, if the subsequent key event
+  // also makes callback, the second callback will be called in the timimg of
+  // the first callback.
+  mozc::commands::Output::Callback callback_command_;
 }
 
-// candidateClicked: is called when the user clicks a candidate item
-// in candidate windows.
-- (void)candidateClicked:(int)id;
+// sendCommand: is called to send SessionCommand to the server
+// from the renderer, when the user clicks a candidate item
+// in candidate windows or when the renderer sends the usage stats
+// event information.
+- (void)sendCommand:(const mozc::commands::SessionCommand&)command;
 
 // reconversionClicked: is called when the user clicks "Reconversion"
 // menu item.
@@ -133,9 +148,13 @@ class RendererInterface;
 // menu item.
 - (IBAction)registerWordClicked:(id)sender;
 
-// characterPadClicked: is called when the user clicks "Character Pad..."
+// characterPaletteClicked: is called when the user clicks
+// "Character Palette..." menu item.
+- (IBAction)characterPaletteClicked:(id)sender;
+
+// handWritingClicked: is called when the user clicks "Hand Wrinting..."
 // menu item.
-- (IBAction)characterPadClicked:(id)sender;
+- (IBAction)handWritingClicked:(id)sender;
 
 // aboutDialogClicked: is called when the user clicks "About Mozc..."
 // menu item.
