@@ -33,26 +33,220 @@
 #include "net/json_util.h"
 
 #include "base/base.h"
-#include "base/testing_util.h"
 #include "base/util.h"
 #include "net/jsoncpp.h"
 #include "net/json_util_test.pb.h"
 #include "testing/base/public/gunit.h"
+#include "testing/base/public/testing_util.h"
 
 namespace mozc {
 namespace net {
+namespace {
+const char * const kDefaultJsonString =
+    "{"
+    "  \"repeated_double_value\": [],"
+    "  \"repeated_float_value\": [],"
+    "  \"repeated_int32_value\": [],"
+    "  \"repeated_int64_value\": [],"
+    "  \"repeated_uint32_value\": [],"
+    "  \"repeated_uint64_value\": [],"
+    "  \"repeated_sint32_value\": [],"
+    "  \"repeated_sint64_value\": [],"
+    "  \"repeated_fixed32_value\": [],"
+    "  \"repeated_fixed64_value\": [],"
+    "  \"repeated_sfixed32_value\": [],"
+    "  \"repeated_sfixed64_value\": [],"
+    "  \"repeated_bool_value\": [],"
+    "  \"repeated_string_value\": [],"
+    "  \"repeated_bytes_value\": [],"
+    "  \"repeated_enum_value\": [],"
+    "  \"repeated_innerenum_value\": [],"
+    "  \"required_double_value\": 201.0,"
+    "  \"required_float_value\": 202.0,"
+    "  \"required_int32_value\": 203,"
+    "  \"required_int64_value\": \"204\","
+    "  \"required_uint32_value\": 205,"
+    "  \"required_uint64_value\": \"206\","
+    "  \"required_sint32_value\": 207,"
+    "  \"required_sint64_value\": \"208\","
+    "  \"required_fixed32_value\": 209,"
+    "  \"required_fixed64_value\": \"210\","
+    "  \"required_sfixed32_value\": 211,"
+    "  \"required_sfixed64_value\": \"212\","
+    "  \"required_bool_value\": true,"
+    "  \"required_string_value\": \"TEST\","
+    "  \"required_bytes_value\": \"TEST\","
+    "  \"required_enum_value\": \"ENUM_C\","
+    "  \"required_innerenum_value\": \"ENUM_2\","
+    "  \"repeated_sub_message\": [],"
+    "  \"required_sub_message\": {"
+    "    \"repeated_double_value\": [],"
+    "    \"repeated_float_value\": [],"
+    "    \"repeated_int32_value\": [],"
+    "    \"repeated_int64_value\": [],"
+    "    \"repeated_uint32_value\": [],"
+    "    \"repeated_uint64_value\": [],"
+    "    \"repeated_sint32_value\": [],"
+    "    \"repeated_sint64_value\": [],"
+    "    \"repeated_fixed32_value\": [],"
+    "    \"repeated_fixed64_value\": [],"
+    "    \"repeated_sfixed32_value\": [],"
+    "    \"repeated_sfixed64_value\": [],"
+    "    \"repeated_bool_value\": [],"
+    "    \"repeated_string_value\": [],"
+    "    \"repeated_bytes_value\": [],"
+    "    \"repeated_enum_value\": [],"
+    "    \"required_double_value\": 201.0,"
+    "    \"required_float_value\": 202.0,"
+    "    \"required_int32_value\": 203,"
+    "    \"required_int64_value\": \"204\","
+    "    \"required_uint32_value\": 205,"
+    "    \"required_uint64_value\": \"206\","
+    "    \"required_sint32_value\": 207,"
+    "    \"required_sint64_value\": \"208\","
+    "    \"required_fixed32_value\": 209,"
+    "    \"required_fixed64_value\": \"210\","
+    "    \"required_sfixed32_value\": 211,"
+    "    \"required_sfixed64_value\": \"212\","
+    "    \"required_bool_value\": true,"
+    "    \"required_string_value\": \"TEST\","
+    "    \"required_bytes_value\": \"TEST\","
+    "    \"required_enum_value\": \"ENUM_C\","
+    "  }"
+    "}";
+const char * const kDefaultSubMessageJsonString =
+    "{"
+    "  \"repeated_double_value\": [],"
+    "  \"repeated_float_value\": [],"
+    "  \"repeated_int32_value\": [],"
+    "  \"repeated_int64_value\": [],"
+    "  \"repeated_uint32_value\": [],"
+    "  \"repeated_uint64_value\": [],"
+    "  \"repeated_sint32_value\": [],"
+    "  \"repeated_sint64_value\": [],"
+    "  \"repeated_fixed32_value\": [],"
+    "  \"repeated_fixed64_value\": [],"
+    "  \"repeated_sfixed32_value\": [],"
+    "  \"repeated_sfixed64_value\": [],"
+    "  \"repeated_bool_value\": [],"
+    "  \"repeated_string_value\": [],"
+    "  \"repeated_bytes_value\": [],"
+    "  \"repeated_enum_value\": [],"
+    "  \"required_double_value\": 201.0,"
+    "  \"required_float_value\": 202.0,"
+    "  \"required_int32_value\": 203,"
+    "  \"required_int64_value\": \"204\","
+    "  \"required_uint32_value\": 205,"
+    "  \"required_uint64_value\": \"206\","
+    "  \"required_sint32_value\": 207,"
+    "  \"required_sint64_value\": \"208\","
+    "  \"required_fixed32_value\": 209,"
+    "  \"required_fixed64_value\": \"210\","
+    "  \"required_sfixed32_value\": 211,"
+    "  \"required_sfixed64_value\": \"212\","
+    "  \"required_bool_value\": true,"
+    "  \"required_string_value\": \"TEST\","
+    "  \"required_bytes_value\": \"TEST\","
+    "  \"required_enum_value\": \"ENUM_C\","
+    "}";
+
+Json::Value GetDefaultExpectedValue() {
+  Json::Value expected_value(Json::objectValue);
+  Json::Reader().parse(kDefaultJsonString, expected_value);
+  return expected_value;
+}
+
+Json::Value GetDefaultSubMessageExpectedValue() {
+  Json::Value expected_value(Json::objectValue);
+  Json::Reader().parse(kDefaultSubMessageJsonString, expected_value);
+  return expected_value;
+}
+
+void FillRequiredFields(protobuf::Message *message) {
+  const protobuf::Descriptor *descriptor = message->GetDescriptor();
+  const protobuf::Reflection *reflection = message->GetReflection();
+  const int field_count = descriptor->field_count();
+  for (size_t i = 0; i < field_count; ++i) {
+    const protobuf::FieldDescriptor *field = descriptor->field(i);
+    if (field->is_repeated()) {
+      const int count = reflection->FieldSize(*message, field);
+      if (field->cpp_type() == protobuf::FieldDescriptor::CPPTYPE_MESSAGE) {
+        for (int j = 0; j < count; ++j) {
+          FillRequiredFields(
+              reflection->MutableRepeatedMessage(message, field, j));
+        }
+      }
+    } else if (reflection->HasField(*message, field)) {
+      if (field->cpp_type() == protobuf::FieldDescriptor::CPPTYPE_MESSAGE) {
+       FillRequiredFields(reflection->MutableMessage(message, field, NULL));
+      }
+    } else if (field->is_required()) {
+      switch (field->cpp_type()) {
+        case protobuf::FieldDescriptor::CPPTYPE_INT32: {
+          reflection->SetInt32(message, field, field->default_value_int32());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_INT64: {
+          reflection->SetInt64(message, field, field->default_value_int64());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_UINT32: {
+          reflection->SetUInt32(message, field, field->default_value_uint32());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_UINT64: {
+          reflection->SetUInt64(message, field, field->default_value_uint64());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_DOUBLE: {
+          reflection->SetDouble(message, field, field->default_value_double());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_FLOAT: {
+          reflection->SetFloat(message, field, field->default_value_float());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_BOOL: {
+          reflection->SetBool(message, field, field->default_value_bool());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_ENUM: {
+          reflection->SetEnum(message, field, field->default_value_enum());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_STRING: {
+          reflection->SetString(message, field, field->default_value_string());
+          break;
+        }
+        case protobuf::FieldDescriptor::CPPTYPE_MESSAGE: {
+          FillRequiredFields(reflection->MutableMessage(message, field, NULL));
+          break;
+        }
+        default: {
+          DLOG(ERROR) << "Unknown or unsupported type: " << field->cpp_type();
+          return;
+        }
+      }
+    }
+  }
+}
+
+}  // namespace
 
 TEST(JsonUtilTest, EmptyTest) {
   TestMsg msg;
   Json::Value json_value;
   EXPECT_TRUE(JsonUtil::ProtobufMessageToJsonValue(msg, &json_value));
-  Json::Value expected_value(Json::objectValue);
-  EXPECT_EQ(expected_value, json_value);
+  Json::Value expected_value = GetDefaultExpectedValue();
+  EXPECT_EQ(Json::FastWriter().write(expected_value),
+            Json::FastWriter().write(json_value));
   TestMsg new_msg;
+  FillRequiredFields(&msg);
   EXPECT_TRUE(JsonUtil::JsonValueToProtobufMessage(json_value, &new_msg));
   EXPECT_PROTO_EQ(msg, new_msg);
 }
 
+
 TEST(JsonUtilTest, ConvertItemTest) {
 #define TEST_CONVERT_ITEM(proto_setter, proto_value, json_name, json_value) \
   { \
@@ -60,11 +254,13 @@ TEST(JsonUtilTest, ConvertItemTest) {
     msg.proto_setter(proto_value); \
     Json::Value value; \
     EXPECT_TRUE(JsonUtil::ProtobufMessageToJsonValue(msg, &value)); \
-    Json::Value expected_value(Json::objectValue); \
+    Json::Value expected_value = GetDefaultExpectedValue(); \
     expected_value[json_name] = json_value; \
-    EXPECT_EQ(expected_value, value); \
+    EXPECT_EQ(Json::FastWriter().write(expected_value), \
+              Json::FastWriter().write(value)); \
     TestMsg new_msg; \
     EXPECT_TRUE(JsonUtil::JsonValueToProtobufMessage(value, &new_msg)); \
+    FillRequiredFields(&msg); \
     EXPECT_PROTO_EQ(msg, new_msg); \
   }
   TEST_CONVERT_ITEM(set_double_value, 1.0, "double_value", 1.0)
@@ -124,6 +320,83 @@ TEST(JsonUtilTest, ConvertItemTest) {
   TEST_CONVERT_ITEM(set_enum_value, ENUM_A, "enum_value", "ENUM_A")
   TEST_CONVERT_ITEM(set_innerenum_value, TestMsg::ENUM_1,
                     "innerenum_value", "ENUM_1")
+
+
+  TEST_CONVERT_ITEM(set_required_double_value, 1.0,
+                    "required_double_value", 1.0)
+  TEST_CONVERT_ITEM(set_required_float_value, 2.0,
+                      "required_float_value", 2.0)
+  TEST_CONVERT_ITEM(set_required_int32_value, 3,
+                    "required_int32_value", Json::Int(3))
+  TEST_CONVERT_ITEM(set_required_int32_value, -3,
+                    "required_int32_value", Json::Int(-3))
+  TEST_CONVERT_ITEM(set_required_int32_value, kint32min,
+                    "required_int32_value", Json::Int(kint32min))
+  TEST_CONVERT_ITEM(set_required_int32_value, kint32max,
+                    "required_int32_value", Json::Int(kint32max))
+  TEST_CONVERT_ITEM(set_required_int64_value, 4, "required_int64_value", "4")
+  TEST_CONVERT_ITEM(set_required_int64_value, -4, "required_int64_value", "-4")
+  TEST_CONVERT_ITEM(set_required_int64_value, kint64min,
+                    "required_int64_value", "-9223372036854775808")
+  TEST_CONVERT_ITEM(set_required_int64_value, kint64max,
+                    "required_int64_value", "9223372036854775807")
+  TEST_CONVERT_ITEM(set_required_uint32_value, 5,
+                    "required_uint32_value", Json::UInt(5))
+  TEST_CONVERT_ITEM(set_required_uint32_value, kuint32max,
+                    "required_uint32_value", Json::UInt(kuint32max))
+  TEST_CONVERT_ITEM(set_required_uint64_value, 6, "required_uint64_value", "6")
+  TEST_CONVERT_ITEM(set_required_uint64_value, kuint64max,
+                    "required_uint64_value", "18446744073709551615")
+  TEST_CONVERT_ITEM(set_required_sint32_value, 7,
+                    "required_sint32_value", Json::Int(7))
+  TEST_CONVERT_ITEM(set_required_sint32_value, -7,
+                    "required_sint32_value", Json::Int(-7))
+  TEST_CONVERT_ITEM(set_required_sint32_value, kint32min,
+                    "required_sint32_value", Json::Int(kint32min))
+  TEST_CONVERT_ITEM(set_required_sint32_value, kint32max,
+                    "required_sint32_value", Json::Int(kint32max))
+  TEST_CONVERT_ITEM(set_required_sint64_value, 8, "required_sint64_value", "8")
+  TEST_CONVERT_ITEM(set_required_sint64_value, -8,
+                    "required_sint64_value", "-8")
+  TEST_CONVERT_ITEM(set_required_sint64_value, kint64min,
+                    "required_sint64_value", "-9223372036854775808")
+  TEST_CONVERT_ITEM(set_required_sint64_value, kint64max,
+                    "required_sint64_value", "9223372036854775807")
+  TEST_CONVERT_ITEM(set_required_fixed32_value, 9,
+                    "required_fixed32_value", Json::UInt(9))
+  TEST_CONVERT_ITEM(set_required_fixed32_value, kuint32max,
+                    "required_fixed32_value", Json::UInt(kuint32max))
+  TEST_CONVERT_ITEM(set_required_fixed64_value, 10,
+                    "required_fixed64_value", "10")
+  TEST_CONVERT_ITEM(set_required_fixed64_value, kuint64max,
+                    "required_fixed64_value", "18446744073709551615")
+  TEST_CONVERT_ITEM(set_required_sfixed32_value, 11,
+                    "required_sfixed32_value", Json::Int(11))
+  TEST_CONVERT_ITEM(set_required_sfixed32_value, -11,
+                    "required_sfixed32_value", Json::Int(-11))
+  TEST_CONVERT_ITEM(set_required_sfixed32_value, kint32min,
+                    "required_sfixed32_value", Json::Int(kint32min))
+  TEST_CONVERT_ITEM(set_required_sfixed32_value, kint32max,
+                    "required_sfixed32_value", Json::Int(kint32max))
+  TEST_CONVERT_ITEM(set_required_sfixed64_value, 12,
+                    "required_sfixed64_value", "12")
+  TEST_CONVERT_ITEM(set_required_sfixed64_value, -12,
+                    "required_sfixed64_value", "-12")
+  TEST_CONVERT_ITEM(set_required_sfixed64_value, kint64min,
+                    "required_sfixed64_value", "-9223372036854775808")
+  TEST_CONVERT_ITEM(set_required_sfixed64_value, kint64max,
+                    "required_sfixed64_value", "9223372036854775807")
+  TEST_CONVERT_ITEM(set_required_bool_value, true, "required_bool_value", true)
+  TEST_CONVERT_ITEM(set_required_bool_value, false,
+                    "required_bool_value", false)
+  TEST_CONVERT_ITEM(set_required_string_value, "string",
+                    "required_string_value", "string")
+  TEST_CONVERT_ITEM(set_required_bytes_value, "bytes",
+                    "required_bytes_value", "bytes")
+  TEST_CONVERT_ITEM(set_required_enum_value, ENUM_A,
+                    "required_enum_value", "ENUM_A")
+  TEST_CONVERT_ITEM(set_required_innerenum_value, TestMsg::ENUM_1,
+                    "required_innerenum_value", "ENUM_1")
 #undef TEST_CONVERT_ITEM
 }
 
@@ -138,13 +411,15 @@ TEST(JsonUtilTest, ConvertRepeatedItemTest) {
     msg.proto_adder(proto_value3); \
     Json::Value value; \
     EXPECT_TRUE(JsonUtil::ProtobufMessageToJsonValue(msg, &value)); \
-    Json::Value expected_value(Json::objectValue); \
+    Json::Value expected_value = GetDefaultExpectedValue(); \
     expected_value[json_name].append(json_value1); \
     expected_value[json_name].append(json_value2); \
     expected_value[json_name].append(json_value3); \
-    EXPECT_EQ(expected_value, value); \
+    EXPECT_EQ(Json::FastWriter().write(expected_value), \
+              Json::FastWriter().write(value)); \
     TestMsg new_msg; \
     EXPECT_TRUE(JsonUtil::JsonValueToProtobufMessage(value, &new_msg)); \
+    FillRequiredFields(&msg); \
     EXPECT_PROTO_EQ(msg, new_msg); \
   }
   TEST_CONVERT_REPEATED_ITEM(add_repeated_double_value, 1.0, 2.0, 3.0,
@@ -285,12 +560,15 @@ TEST(JsonUtilTest, SubMsgTest) {
     sub_msg->set_float_value(200.0);
     Json::Value value;
     EXPECT_TRUE(JsonUtil::ProtobufMessageToJsonValue(msg, &value));
-    Json::Value expected_value(Json::objectValue);
+    Json::Value expected_value = GetDefaultExpectedValue();
+    expected_value["sub_message"] = GetDefaultSubMessageExpectedValue();
     expected_value["sub_message"]["double_value"] = 100.0;
     expected_value["sub_message"]["float_value"] = 200.0;
-    EXPECT_EQ(expected_value, value);
+    EXPECT_EQ(Json::FastWriter().write(expected_value),
+              Json::FastWriter().write(value));
     TestMsg new_msg;
     EXPECT_TRUE(JsonUtil::JsonValueToProtobufMessage(value, &new_msg));
+    FillRequiredFields(&msg);
     EXPECT_PROTO_EQ(msg, new_msg);
   }
   {
@@ -304,10 +582,10 @@ TEST(JsonUtilTest, SubMsgTest) {
     sub_msg3->add_repeated_bool_value(false);
     Json::Value value;
     EXPECT_TRUE(JsonUtil::ProtobufMessageToJsonValue(msg, &value));
-    Json::Value expected_value(Json::objectValue);
-    Json::Value sub_value1(Json::objectValue);
-    Json::Value sub_value2(Json::objectValue);
-    Json::Value sub_value3(Json::objectValue);
+    Json::Value expected_value = GetDefaultExpectedValue();
+    Json::Value sub_value1 = GetDefaultSubMessageExpectedValue();
+    Json::Value sub_value2 = GetDefaultSubMessageExpectedValue();
+    Json::Value sub_value3 = GetDefaultSubMessageExpectedValue();
     sub_value1["double_value"] = 100.0;
     sub_value3["float_value"] = 200.0;
     sub_value3["repeated_bool_value"].append(true);
@@ -315,9 +593,11 @@ TEST(JsonUtilTest, SubMsgTest) {
     expected_value["repeated_sub_message"].append(sub_value1);
     expected_value["repeated_sub_message"].append(sub_value2);
     expected_value["repeated_sub_message"].append(sub_value3);
-    EXPECT_EQ(expected_value, value);
+    EXPECT_EQ(Json::FastWriter().write(expected_value),
+              Json::FastWriter().write(value));
     TestMsg new_msg;
     EXPECT_TRUE(JsonUtil::JsonValueToProtobufMessage(value, &new_msg));
+    FillRequiredFields(&msg);
     EXPECT_PROTO_EQ(msg, new_msg);
   }
 }
@@ -340,13 +620,14 @@ TEST(JsonUtilTest, CombinedTest) {
 
   Json::Value value;
   EXPECT_TRUE(JsonUtil::ProtobufMessageToJsonValue(msg, &value));
-  Json::Value expected_value(Json::objectValue);
-  Json::Value sub_value(Json::objectValue);
-  Json::Value repeated_sub_value1(Json::objectValue);
-  Json::Value repeated_sub_value2(Json::objectValue);
+  Json::Value expected_value = GetDefaultExpectedValue();
+  Json::Value sub_value = GetDefaultSubMessageExpectedValue();
+  Json::Value repeated_sub_value1 = GetDefaultSubMessageExpectedValue();
+  Json::Value repeated_sub_value2 = GetDefaultSubMessageExpectedValue();
   expected_value["double_value"] = 1.0;
   expected_value["float_value"] = 2.0;
   expected_value["int32_value"] = Json::Int(3);
+  expected_value["sub_message"] = GetDefaultSubMessageExpectedValue();
   expected_value["sub_message"]["string_value"] = "123";
   expected_value["sub_message"]["repeated_int32_value"].append(Json::Int(10));
   expected_value["sub_message"]["repeated_int32_value"].append(Json::Int(20));
@@ -356,9 +637,11 @@ TEST(JsonUtilTest, CombinedTest) {
   repeated_sub_value2["repeated_enum_value"].append("ENUM_C");
   repeated_sub_value2["repeated_enum_value"].append("ENUM_A");
   expected_value["repeated_sub_message"].append(repeated_sub_value2);
-  EXPECT_EQ(expected_value, value);
+  EXPECT_EQ(Json::FastWriter().write(expected_value),
+            Json::FastWriter().write(value));
   TestMsg new_msg;
   EXPECT_TRUE(JsonUtil::JsonValueToProtobufMessage(value, &new_msg));
+  FillRequiredFields(&msg);
   EXPECT_PROTO_EQ(msg, new_msg);
 }
 
