@@ -33,7 +33,7 @@
 #define MOZC_REWRITER_COLLOCATION_UTIL_H_
 
 #include <string>
-#include "base/base.h"
+#include "base/port.h"
 #include "base/string_piece.h"
 
 namespace mozc {
@@ -55,6 +55,8 @@ class CollocationUtil {
   // Removes characters for normalizing.
   static void RemoveExtraCharacters(
       const StringPiece input, bool remove_number, string *output);
+
+  DISALLOW_IMPLICIT_CONSTRUCTORS(CollocationUtil);
 };
 
 }  // namespace mozc
