@@ -32,7 +32,6 @@
 #include <string>
 
 #include "base/scoped_ptr.h"
-#include "base/testing_util.h"
 #include "composer/composer.h"
 #include "composer/table.h"
 #include "converter/converter_interface.h"
@@ -42,6 +41,7 @@
 #include "session/session_converter.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
+#include "testing/base/public/testing_util.h"
 
 namespace mozc {
 namespace session {
