@@ -39,10 +39,9 @@
 #include "config/config.pb.h"
 #include "config/config_handler.h"
 #include "converter/converter_interface.h"
-#include "engine/engine_interface.h"
+#include "engine/mock_data_engine_factory.h"
 #include "engine/user_data_manager_interface.h"
 #include "session/commands.pb.h"
-#include "session/japanese_session_factory.h"
 #include "session/key_parser.h"
 #include "session/request_test_util.h"
 #include "session/session_handler_test_util.h"
@@ -72,22 +71,25 @@ using mozc::commands::Request;
 using mozc::commands::RequestForUnitTest;
 using mozc::config::Config;
 using mozc::config::ConfigHandler;
+using mozc::MockDataEngineFactory;
+using mozc::EngineInterface;
 using mozc::protobuf::FieldDescriptor;
 using mozc::protobuf::Message;
 using mozc::protobuf::TextFormat;
-using mozc::session::testing::JapaneseSessionHandlerTestBase;
+using mozc::session::testing::SessionHandlerTestBase;
 using mozc::session::testing::TestSessionClient;
 using testing::WithParamInterface;
 
-class SessionHandlerScenarioTest : public JapaneseSessionHandlerTestBase,
+class SessionHandlerScenarioTest : public SessionHandlerTestBase,
                                    public WithParamInterface<const char *> {
  protected:
   virtual void SetUp() {
     // Note that singleton Config instance is backed up and restored
-    // by JapaneseSessionHandlerTestBase's SetUp and TearDown methods.
-    JapaneseSessionHandlerTestBase::SetUp();
+    // by SessionHandlerTestBase's SetUp and TearDown methods.
+    SessionHandlerTestBase::SetUp();
 
-    client_.reset(new TestSessionClient);
+    engine_.reset(MockDataEngineFactory::Create());
+    client_.reset(new TestSessionClient(engine_.get()));
     config_.reset(new Config);
     last_output_.reset(new Output);
     request_.reset(new Request);
@@ -108,13 +110,14 @@ class SessionHandlerScenarioTest : public JapaneseSessionHandlerTestBase,
 
   void ClearUserPrediction() {
     EXPECT_TRUE(client_->ClearUserPrediction());
-    EXPECT_TRUE(engine()->GetUserDataManager()->WaitForSyncerForTest());
+    EXPECT_TRUE(engine_->GetUserDataManager()->WaitForSyncerForTest());
   }
 
   void ClearUsageStats() {
     mozc::usage_stats::UsageStats::ClearAllStatsForTest();
   }
 
+  scoped_ptr<EngineInterface> engine_;
   scoped_ptr<TestSessionClient> client_;
   scoped_ptr<Config> config_;
   scoped_ptr<Output> last_output_;
@@ -123,33 +126,47 @@ class SessionHandlerScenarioTest : public JapaneseSessionHandlerTestBase,
 
 // Tests should be passed.
 const char *kScenarioFileList[] = {
-  "data/test/session/scenario/auto_partial_suggestion.txt",
-  "data/test/session/scenario/b7132535_scenario.txt",
-  "data/test/session/scenario/b7321313_scenario.txt",
-  "data/test/session/scenario/b8703702_scenario.txt",
-  "data/test/session/scenario/change_request.txt",
-  "data/test/session/scenario/clear_user_prediction.txt",
-  "data/test/session/scenario/composition_display_as.txt",
-  "data/test/session/scenario/conversion.txt",
-  "data/test/session/scenario/conversion_display_as.txt",
-  "data/test/session/scenario/conversion_with_history_segment.txt",
-  "data/test/session/scenario/conversion_with_long_history_segments.txt",
-  "data/test/session/scenario/delete_history.txt",
-  "data/test/session/scenario/desktop_t13n_candidates.txt",
-  "data/test/session/scenario/insert_characters.txt",
-  "data/test/session/scenario/mobile_qwerty_transliteration_scenario.txt",
-  "data/test/session/scenario/mobile_t13n_candidates.txt",
-  "data/test/session/scenario/on_off_cancel.txt",
-  "data/test/session/scenario/partial_suggestion.txt",
-  "data/test/session/scenario/pending_character.txt",
-  "data/test/session/scenario/predict_and_convert.txt",
-  "data/test/session/scenario/reconvert.txt",
-  "data/test/session/scenario/revert.txt",
-  "data/test/session/scenario/segment_focus.txt",
-  "data/test/session/scenario/segment_width.txt",
-  "data/test/session/scenario/twelvekeys_switch_inputmode_scenario.txt",
-  "data/test/session/scenario/twelvekeys_toggle_hiragana_preedit_scenario.txt",
-  "data/test/session/scenario/undo.txt",
+#define DATA_DIR "data/test/session/scenario/"
+  DATA_DIR "auto_partial_suggestion.txt",
+  DATA_DIR "b7132535_scenario.txt",
+  DATA_DIR "b7321313_scenario.txt",
+  DATA_DIR "b8703702_scenario.txt",
+  DATA_DIR "change_request.txt",
+  DATA_DIR "clear_user_prediction.txt",
+  DATA_DIR "commit.txt",
+  DATA_DIR "composition_display_as.txt",
+  DATA_DIR "conversion.txt",
+  DATA_DIR "conversion_display_as.txt",
+  DATA_DIR "conversion_with_history_segment.txt",
+  DATA_DIR "conversion_with_long_history_segments.txt",
+  DATA_DIR "convert_from_full_ascii_to_t13n.txt",
+  DATA_DIR "convert_from_full_katakana_to_t13n.txt",
+  DATA_DIR "convert_from_half_ascii_to_t13n.txt",
+  DATA_DIR "convert_from_half_katakana_to_t13n.txt",
+  DATA_DIR "convert_from_hiragana_to_t13n.txt",
+  DATA_DIR "delete_history.txt",
+  DATA_DIR "desktop_t13n_candidates.txt",
+#if !defined(OS_MACOSX) && !defined(OS_ANDROID)
+  // "InputModeX" commands are not supported on Mac and Android.
+  // Mac: We do not have the way to change the mode indicator from IME.
+  // Android: Input mode change is performed by changing software keyboard.
+  DATA_DIR "input_mode.txt",
+#endif
+  DATA_DIR "insert_characters.txt",
+  DATA_DIR "mobile_qwerty_transliteration_scenario.txt",
+  DATA_DIR "mobile_t13n_candidates.txt",
+  DATA_DIR "on_off_cancel.txt",
+  DATA_DIR "partial_suggestion.txt",
+  DATA_DIR "pending_character.txt",
+  DATA_DIR "predict_and_convert.txt",
+  DATA_DIR "reconvert.txt",
+  DATA_DIR "revert.txt",
+  DATA_DIR "segment_focus.txt",
+  DATA_DIR "segment_width.txt",
+  DATA_DIR "twelvekeys_switch_inputmode_scenario.txt",
+  DATA_DIR "twelvekeys_toggle_hiragana_preedit_scenario.txt",
+  DATA_DIR "undo.txt",
+#undef DATA_DIR
 };
 
 INSTANTIATE_TEST_CASE_P(SessionHandlerScenarioParameters,
@@ -166,6 +183,7 @@ const char *kUsageStatsScenarioFileList[] = {
   DATA_DIR "continuous_input.txt",
   DATA_DIR "conversion.txt",
   DATA_DIR "insert_space.txt",
+  DATA_DIR "language_aware_input.txt",
   DATA_DIR "mouse_select_from_suggestion.txt",
   DATA_DIR "multiple_backspace_after_commit.txt",
   DATA_DIR "multiple_segments.txt",
@@ -429,6 +447,11 @@ TEST_P(SessionHandlerScenarioTest, TestImpl) {
     } else if (command == "SET_MOBILE_REQUEST") {
       RequestForUnitTest::FillMobileRequest(request_.get());
       ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
+    } else if (command == "SET_REQUEST") {
+      ASSERT_EQ(3, columns.size());
+      ASSERT_TRUE(SetOrAddFieldValueFromString(columns[1], columns[2],
+                                               request_.get()));
+      ASSERT_TRUE(client_->SetRequest(*request_, last_output_.get()));
     } else if (command == "SET_CONFIG") {
       ASSERT_EQ(3, columns.size());
       ASSERT_TRUE(SetOrAddFieldValueFromString(columns[1], columns[2],
@@ -472,7 +495,9 @@ TEST_P(SessionHandlerScenarioTest, TestImpl) {
       for (int i = 0; i < preedit.segment_size(); ++i) {
         preedit_string += preedit.segment(i).value();
       }
-      EXPECT_EQ(expected_preedit, preedit_string) << preedit.Utf8DebugString();
+      EXPECT_EQ(expected_preedit, preedit_string)
+          << "Expected preedit: " << expected_preedit << "\n"
+          << "Actual preedit: " <<preedit.Utf8DebugString();
     } else if (command == "EXPECT_PREEDIT_IN_DETAIL") {
       ASSERT_LE(1, columns.size());
       const mozc::commands::Preedit &preedit = last_output_->preedit();
