@@ -45,8 +45,10 @@
 
 #include "base/system_util.h"
 #include "base/win_util.h"
+#include "win32/base/text_icon.h"
 #include "win32/base/tsf_profile.h"
 #include "win32/tip/tip_dll_module.h"
+#include "win32/tip/tip_resource.h"
 
 namespace mozc {
 namespace win32 {
@@ -69,6 +71,35 @@ const int kTipLangBarMenuCookie = (('M' << 24) |
                                    ('z' << 8) |
                                    ('c' << 0));
 
+// "ＭＳ ゴシック"
+const char kTextIconFont[] =
+    "\xEF\xBC\xAD\xEF\xBC\xB3"
+    "\x20\xE3\x82\xB4\xE3\x82\xB7\xE3\x83\x83\xE3\x82\xAF";
+
+// TODO(yukawa): Refactor LangBar code so that we can configure following
+// settings as a part of initialization.
+string GetIconStringIfNecessary(UINT icon_id) {
+  switch (icon_id) {
+    case IDI_DIRECT_NT:
+      return "A";
+    case IDI_HIRAGANA_NT:
+      // "あ"
+      return "\xE3\x81\x82";
+    case IDI_FULL_KATAKANA_NT:
+      // "ア"
+      return "\xE3\x82\xA2";
+    case IDI_HALF_ALPHANUMERIC_NT:
+      return "_A";
+    case IDI_FULL_ALPHANUMERIC_NT:
+      // "Ａ"
+      return "\xEF\xBC\xA1";
+    case IDI_HALF_KATAKANA_NT:
+      // "_ｱ"
+      return "_" "\xEF\xBD\xB1";
+  }
+  return string();
+}
+
 // Loads an icon which is appropriate for the current theme.
 // An icon ID 0 represents "no icon".
 HICON LoadIconFromResource(HINSTANCE instance,
@@ -100,15 +131,19 @@ HICON LoadIconFromResource(HINSTANCE instance,
     return nullptr;
   }
 
-  CDC desktop_dc(::GetDC(nullptr));
-  const int dpi_x = desktop_dc.GetDeviceCaps(LOGPIXELSX);
-  const int dpi_y = desktop_dc.GetDeviceCaps(LOGPIXELSY);
-  const int icon_width = ::MulDiv(16, dpi_x, kDefaultDPI);
-  const int icon_height = ::MulDiv(16, dpi_y, kDefaultDPI);
+  const auto icon_size = ::GetSystemMetrics(SM_CYSMICON);
+
+  // Replace some text icons with on-the-fly image drawn with MS-Gothic.
+  const auto &icon_text = GetIconStringIfNecessary(id);
+  if (!icon_text.empty()) {
+    const COLORREF text_color = ::GetSysColor(COLOR_WINDOWTEXT);
+    return TextIcon::CreateMonochromeIcon(
+        icon_size, icon_size, icon_text, kTextIconFont, text_color);
+  }
 
   return static_cast<HICON>(::LoadImage(
       instance, MAKEINTRESOURCE(id),
-      IMAGE_ICON, icon_width, icon_height, LR_CREATEDIBSECTION));
+      IMAGE_ICON, icon_size, icon_size, LR_CREATEDIBSECTION));
 }
 
 // Retrieves the bitmap handle loaded by using an icon ID.
@@ -315,31 +350,39 @@ STDMETHODIMP TipLangBarButton::OnClick(TfLBIClick click,
       info.dwTypeData = data->text_;
 
       switch (data->flags_) {
-      case TF_LBMENUF_RADIOCHECKED:
-        info.fMask |= MIIM_STATE;
-        info.fState |= MFS_CHECKED;
-        info.fMask |= MIIM_FTYPE;
-        info.fType |= MFT_RADIOCHECK;
-        break;
-      case TF_LBMENUF_CHECKED:
-        info.fMask |= MIIM_STATE;
-        info.fState |= MFS_CHECKED;
-        break;
-      case TF_LBMENUF_SUBMENU:
-        // TODO(yukawa): Support this.
-        break;
-      case TF_LBMENUF_GRAYED:
-        info.fMask |= MIIM_STATE;
-        info.fState |= MFS_GRAYED;
-        break;
+        case TF_LBMENUF_RADIOCHECKED:
+          info.fMask |= MIIM_STATE;
+          info.fState |= MFS_CHECKED;
+          info.fMask |= MIIM_FTYPE;
+          info.fType |= MFT_RADIOCHECK;
+          break;
+        case TF_LBMENUF_CHECKED:
+          info.fMask |= MIIM_STATE;
+          info.fState |= MFS_CHECKED;
+          break;
+        case TF_LBMENUF_SUBMENU:
+          // TODO(yukawa): Support this.
+          break;
+        case TF_LBMENUF_GRAYED:
+          info.fMask |= MIIM_STATE;
+          info.fState |= MFS_GRAYED;
+          break;
+        default:
+          info.fMask |= MIIM_STATE;
+          info.fState |= MFS_ENABLED;
+          break;
       }
     }
     menu.InsertMenuItemW(i, TRUE, &info);
   }
 
+  // Caveats: TPM_NONOTIFY is important because the attached window may
+  // change the menu state unless this flag is specified. We actually suffered
+  // from this issue with Internet Explorer 10 on Windows 8. b/10217103.
+  const DWORD kMenuFlags = TPM_NONOTIFY | TPM_RETURNCMD | TPM_LEFTALIGN |
+                           TPM_TOPALIGN | TPM_LEFTBUTTON;
   const BOOL result = menu.TrackPopupMenu(
-      TPM_RETURNCMD | TPM_LEFTALIGN | TPM_TOPALIGN | TPM_LEFTBUTTON,
-      point.x, point.y, ::GetFocus());
+      kMenuFlags, point.x, point.y, ::GetFocus());
   if (!result) {
     return E_FAIL;
   }
